@@ -136,27 +136,63 @@ static void status_done_locked ()
 }
 
 static void add_file (PlaylistAddItem && item, Playlist::FilterFunc filter,
- void * user, AddResult * result, bool validate)
+ void * user, AddResult * result, bool skip_invalid)
 {
     AUDINFO ("Adding file: %s\n", (const char *) item.filename);
     status_update (item.filename, result->items.len ());
 
-    /* If the item doesn't already have a valid tuple, and isn't a subtune
-     * itself, then probe it to expand any subtunes.  The "validate" check (used
-     * to skip non-audio files when adding folders) is also nested within this
-     * block; note that "validate" is always false for subtunes. */
+    /*
+     * If possible, we'll wait until the file is added to the playlist to probe
+     * it.  There are a couple of reasons why we might need to probe it now:
+     *
+     * 1. We're adding a folder, and need to skip over non-audio files (the
+     *    "skip invalid" flag indicates this case).
+     * 2. The file might have subtunes, which we need to expand in order to add
+     *    them to the playlist correctly.
+     *
+     * If we already have metadata, or the file is itself a subtune, then
+     * neither of these reasons apply.
+     */
     if (! item.tuple.valid () && ! is_subtune (item.filename))
     {
+        /* If we open the file to identify the decoder, we can re-use the same
+         * handle to read metadata. */
         VFSFile file;
 
         if (! item.decoder)
         {
-            bool fast = ! aud_get_bool (nullptr, "slow_probe");
-            item.decoder = aud_file_find_decoder (item.filename, fast, file);
-            if (validate && ! item.decoder)
-                return;
+            if (aud_get_bool (nullptr, "slow_probe"))
+            {
+                /* The slow path.  User settings dictate that we should try to
+                 * find a decoder even if we don't recognize the file extension. */
+                item.decoder = aud_file_find_decoder (item.filename, false, file);
+                if (skip_invalid && ! item.decoder)
+                    return;
+            }
+            else
+            {
+                /* The fast path.  First see whether any plugins recognize the
+                 * file extension.  Note that it's possible for multiple plugins
+                 * to recognize the same extension (.ogg, for example). */
+                int flags = probe_by_filename (item.filename);
+                if (skip_invalid && ! (flags & PROBE_FLAG_HAS_DECODER))
+                    return;
+
+                if ((flags & PROBE_FLAG_MIGHT_HAVE_SUBTUNES))
+                {
+                    /* At least one plugin recognized the file extension and
+                     * indicated that there might be subtunes.  Figure out for
+                     * sure which decoder we need to use for this file. */
+                    item.decoder = aud_file_find_decoder (item.filename, true, file);
+                    if (skip_invalid && ! item.decoder)
+                        return;
+                }
+            }
         }
 
+        /* At this point we've either identified the decoder or determined that
+         * the file doesn't have any subtunes.  If the former, read the tag so
+         * so we can expand any subtunes we find. */
         if (item.decoder && input_plugin_has_subtunes (item.decoder))
             aud_file_read_tag (item.filename, item.decoder, file, item.tuple);
     }
@@ -183,9 +219,9 @@ static void add_file (PlaylistAddItem && item, Playlist::FilterFunc filter,
 /* To prevent infinite recursion, we currently allow adding a folder from within
  * a playlist, but not a playlist from within a folder, nor a second playlist
  * from within a playlist (this last rule is enforced by setting
- * <allow_playlist> to false from within add_playlist()). */
+ * <from_playlist> to true from within add_playlist()). */
 static void add_generic (PlaylistAddItem && item, Playlist::FilterFunc filter,
- void * user, AddResult * result, bool save_title, bool allow_playlist);
+ void * user, AddResult * result, bool save_title, bool from_playlist);
 
 static void add_playlist (const char * filename, Playlist::FilterFunc filter,
  void * user, AddResult * result, bool save_title)
@@ -203,7 +239,7 @@ static void add_playlist (const char * filename, Playlist::FilterFunc filter,
         result->title = title;
 
     for (auto & item : items)
-        add_generic (std::move (item), filter, user, result, false, false);
+        add_generic (std::move (item), filter, user, result, false, true);
 }
 
 static void add_cuesheets (Index<String> & files, Playlist::FilterFunc filter,
@@ -311,14 +347,21 @@ static void add_folder (const char * filename, Playlist::FilterFunc filter,
 
         if (mode & VFS_IS_REGULAR)
             add_file ({String (file)}, filter, user, result, true);
-        else if (mode & VFS_IS_DIR)
+        else if ((mode & VFS_IS_DIR) && aud_get_bool (nullptr, "recurse_folders"))
             add_folder (file, filter, user, result, false);
     }
 }
 
 static void add_generic (PlaylistAddItem && item, Playlist::FilterFunc filter,
- void * user, AddResult * result, bool save_title, bool allow_playlist)
+ void * user, AddResult * result, bool save_title, bool from_playlist)
 {
+    if (! strstr (item.filename, "://"))
+    {
+        /* Let's not add random junk to the playlist. */
+        AUDERR ("Invalid URI: %s\n", (const char *) item.filename);
+        return;
+    }
+
     if (filter && ! filter (item.filename, user))
     {
         result->filtered = true;
@@ -331,11 +374,16 @@ static void add_generic (PlaylistAddItem && item, Playlist::FilterFunc filter,
         add_file (std::move (item), filter, user, result, false);
     else
     {
-        String error;
-        VFSFileTest mode = VFSFile::test_file (item.filename,
-         VFSFileTest (VFS_IS_DIR | VFS_NO_ACCESS), error);
+        int tests = 0;
+        if (! from_playlist)
+            tests |= VFS_NO_ACCESS;
+        if (! from_playlist || aud_get_bool (nullptr, "folders_in_playlist"))
+            tests |= VFS_IS_DIR;
 
-        if (mode & VFS_NO_ACCESS)
+        String error;
+        VFSFileTest mode = VFSFile::test_file (item.filename, (VFSFileTest) tests, error);
+
+        if ((mode & VFS_NO_ACCESS))
             aud_ui_show_error (str_printf (_("Error reading %s:\n%s"),
              (const char *) item.filename, (const char *) error));
         else if (mode & VFS_IS_DIR)
@@ -343,7 +391,7 @@ static void add_generic (PlaylistAddItem && item, Playlist::FilterFunc filter,
             add_folder (item.filename, filter, user, result, save_title);
             result->saw_folder = true;
         }
-        else if (allow_playlist && Playlist::filename_is_playlist (item.filename))
+        else if ((! from_playlist) && Playlist::filename_is_playlist (item.filename))
             add_playlist (item.filename, filter, user, result, save_title);
         else
             add_file (std::move (item), filter, user, result, false);
@@ -474,7 +522,7 @@ static void * add_worker (void * unused)
         bool save_title = (task->items.len () == 1);
 
         for (auto & item : task->items)
-            add_generic (std::move (item), task->filter, task->user, result, save_title, true);
+            add_generic (std::move (item), task->filter, task->user, result, save_title, false);
 
         delete task;
 
