@@ -160,6 +160,14 @@ EXPORT StringBuf str_printf (const char * format, ...)
     return str;
 }
 
+EXPORT void str_append_printf (StringBuf & str, const char * format, ...)
+{
+    va_list args;
+    va_start (args, format);
+    str_append_vprintf (str, format, args);
+    va_end (args);
+}
+
 EXPORT StringBuf str_vprintf (const char * format, va_list args)
 {
     StringBuf str (-1);
@@ -168,6 +176,14 @@ EXPORT StringBuf str_vprintf (const char * format, va_list args)
     return str;
 }
 
+EXPORT void str_append_vprintf (StringBuf & str, const char * format, va_list args)
+{
+    int len0 = str.len ();
+    str.resize (-1);
+    int len1 = vsnprintf (str + len0, str.len () - len0, format, args);
+    str.resize (len0 + len1);
+}
+
 EXPORT bool str_has_prefix_nocase (const char * str, const char * prefix)
 {
     return ! g_ascii_strncasecmp (str, prefix, strlen (prefix));
@@ -585,27 +601,30 @@ EXPORT StringBuf filename_to_uri (const char * name)
      * 1) system locale is not UTF-8, and
      * 2) filename is not already valid UTF-8 */
     if (! g_get_charset (nullptr) && ! g_utf8_validate (name, -1, nullptr))
-        buf.steal (str_from_locale (name));
-
-    if (! buf)
-        buf.steal (str_copy (name));
+        buf = str_from_locale (name);
 #endif
 
-    buf.steal (str_encode_percent (buf));
+    buf = str_encode_percent (buf ? buf : name);
     buf.insert (0, URI_PREFIX);
-    return buf;
+    return buf.settle ();
 }
 
-/* Like g_filename_from_uri, but converts the filename from UTF-8 to the system
- * locale after percent-decoding (except on Windows, where filenames are assumed
- * to be UTF-8).  On Windows, strips the leading '/' and replaces '/' with '\'. */
+/* Like g_filename_from_uri, but optionally converts the filename from UTF-8 to
+ * the system locale after percent-decoding (except on Windows, where filenames
+ * are assumed to be UTF-8).  On Windows, strips the leading '/' and replaces
+ * '/' with '\'.  If the input is not a valid URI, it is assumed to be a local
+ * filename already and is not percent-decoded. */
 
 EXPORT StringBuf uri_to_filename (const char * uri, bool use_locale)
 {
-    if (strncmp (uri, URI_PREFIX, URI_PREFIX_LEN))
-        return StringBuf ();
+    StringBuf buf;
 
-    StringBuf buf = str_decode_percent (uri + URI_PREFIX_LEN);
+    if (! strncmp (uri, URI_PREFIX, URI_PREFIX_LEN))
+        buf = str_decode_percent (uri + URI_PREFIX_LEN);
+    else if (! strstr (uri, "://"))  /* already a local filename? */
+        buf = str_copy (uri);
+    else
+        return StringBuf ();
 
 #ifndef _WIN32
     /* convert to locale if:
@@ -616,19 +635,19 @@ EXPORT StringBuf uri_to_filename (const char * uri, bool use_locale)
     {
         StringBuf locale = str_to_locale (buf);
         if (locale)
-            buf.steal (std::move (locale));
+            buf = std::move (locale);
     }
 #endif
 
     /* if UTF-8 was requested, make sure the result is valid */
     if (! use_locale)
     {
-        buf.steal (str_to_utf8 (std::move (buf)));
+        buf = str_to_utf8 (std::move (buf));
         if (! buf)
             return StringBuf ();
     }
 
-    return filename_normalize (std::move (buf));
+    return filename_normalize (buf.settle ());
 }
 
 /* Formats a URI for human-readable display.  Percent-decodes and, for file://
@@ -739,14 +758,45 @@ EXPORT StringBuf uri_construct (const char * path, const char * reference)
 
     StringBuf buf = str_to_utf8 (path, -1);
     if (! buf)
-        return buf;
+        return StringBuf ();
 
     if (aud_get_bool (nullptr, "convert_backslash"))
         str_replace_char (buf, '\\', '/');
 
-    buf.steal (str_encode_percent (buf));
+    buf = str_encode_percent (buf);
     buf.insert (0, reference, slash + 1 - reference);
-    return buf;
+    return buf.settle ();
+}
+
+/* Basically the reverse of uri_construct().
+ * First try to split off a relative path (if so configured).
+ * Failing that, try to convert to a local filename.
+ * Failing that, return the URI as-is.
+ *
+ * All output is UTF-8 for portability.
+ *
+ * Parameters:
+ *   1. uri: the full URI of a song file
+ *   2. reference: the full URI of the playlist being written */
+
+EXPORT StringBuf uri_deconstruct (const char * uri, const char * reference)
+{
+    if (aud_get_bool (nullptr, "export_relative_paths"))
+    {
+        const char * slash = strrchr (reference, '/');
+        if (slash && ! strncmp (uri, reference, slash + 1 - reference))
+        {
+            StringBuf path = str_to_utf8 (str_decode_percent (uri + (slash + 1 - reference)));
+            if (path)
+                return path;
+        }
+    }
+
+    StringBuf filename = uri_to_filename (uri, false);
+    if (filename)
+        return filename;
+
+    return str_copy (uri);
 }
 
 /* Like strcasecmp, but orders numbers correctly (2 before 10). */
@@ -991,24 +1041,22 @@ EXPORT double str_to_double (const char * string)
     return neg ? -val : val;
 }
 
-EXPORT StringBuf int_to_str (int val)
+EXPORT void str_insert_int (StringBuf & string, int pos, int val)
 {
     bool neg = (val < 0);
     unsigned absval = neg ? -val : val;
 
     int digits = digits_for (absval);
-    StringBuf buf ((neg ? 1 : 0) + digits);
+    int len = (neg ? 1 : 0) + digits;
+    char * set = string.insert (pos, nullptr, len);
 
-    char * set = buf;
     if (neg)
         * (set ++) = '-';
 
     uint_to_str (absval, set, digits);
-
-    return buf;
 }
 
-EXPORT StringBuf double_to_str (double val)
+EXPORT void str_insert_double (StringBuf & string, int pos, double val)
 {
     bool neg = (val < 0);
     if (neg)
@@ -1028,9 +1076,9 @@ EXPORT StringBuf double_to_str (double val)
         decimals --;
 
     int digits = digits_for (i);
-    StringBuf buf ((neg ? 1 : 0) + digits + (decimals ? 1 : 0) + decimals);
+    int len = (neg ? 1 : 0) + digits + (decimals ? 1 : 0) + decimals;
+    char * set = string.insert (pos, nullptr, len);
 
-    char * set = buf;
     if (neg)
         * (set ++) = '-';
 
@@ -1042,7 +1090,19 @@ EXPORT StringBuf double_to_str (double val)
         * (set ++) = '.';
         uint_to_str (f, set, decimals);
     }
+}
 
+EXPORT StringBuf int_to_str (int val)
+{
+    StringBuf buf;
+    str_insert_int (buf, 0, val);
+    return buf;
+}
+
+EXPORT StringBuf double_to_str (double val)
+{
+    StringBuf buf;
+    str_insert_double (buf, 0, val);
     return buf;
 }
 
