@@ -28,6 +28,7 @@
 #include "audstrings.h"
 #include "i18n.h"
 #include "interface.h"
+#include "parse.h"
 #include "plugin.h"
 #include "runtime.h"
 
@@ -35,7 +36,7 @@
 
 /* Increment this when the format of the plugin-registry file changes.
  * Add 10 if the format changes in a way that will break parse_plugins_fallback(). */
-#define FORMAT 10
+#define FORMAT 11
 
 /* Oldest file format supported by parse_plugins_fallback() */
 #define MIN_FORMAT 2  // "enabled" flag was added in Audacious 2.4
@@ -50,7 +51,7 @@ class PluginHandle
 public:
     String basename, path;
     bool loaded;
-    int timestamp, version;
+    int timestamp, version, flags;
     PluginType type;
     Plugin * header;
     String name, domain;
@@ -71,12 +72,13 @@ public:
     int has_subtunes, writes_tag;
 
     PluginHandle (const char * basename, const char * path, bool loaded,
-     int timestamp, int version, PluginType type, Plugin * header) :
+     int timestamp, int version, int flags, PluginType type, Plugin * header) :
         basename (basename),
         path (path),
         loaded (loaded),
         timestamp (timestamp),
         version (version),
+        flags (flags),
         type (type),
         header (header),
         priority (0),
@@ -116,6 +118,7 @@ static constexpr aud::array<InputKey, const char *> input_key_names = {
 static aud::array<PluginType, Index<PluginHandle *>> plugins;
 static aud::array<PluginType, Index<PluginHandle *>> compatible;
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static bool modified = false;
 
 static StringBuf get_basename (const char * path)
 {
@@ -167,6 +170,7 @@ static void plugin_save (PluginHandle * plugin, FILE * handle)
     fprintf (handle, "%s %s\n", plugin_type_names[plugin->type], (const char *) plugin->path);
     fprintf (handle, "stamp %d\n", plugin->timestamp);
     fprintf (handle, "version %d\n", plugin->version);
+    fprintf (handle, "flags %d\n", plugin->flags);
     fprintf (handle, "name %s\n", (const char *) plugin->name);
 
     if (plugin->domain)
@@ -187,6 +191,9 @@ static void plugin_save (PluginHandle * plugin, FILE * handle)
 
 void plugin_registry_save ()
 {
+    if (! modified)
+        return;
+
     FILE * handle = open_registry_file ("w");
     if (! handle)
         return;
@@ -196,103 +203,78 @@ void plugin_registry_save ()
     for (auto & list : plugins)
     {
         for (PluginHandle * plugin : list)
-        {
             plugin_save (plugin, handle);
+    }
+
+    fclose (handle);
+    modified = false;
+}
+
+void plugin_registry_cleanup ()
+{
+    for (auto & list : plugins)
+    {
+        for (PluginHandle * plugin : list)
             delete plugin;
-        }
 
         list.clear ();
     }
 
     for (auto & list : compatible)
         list.clear ();
-
-    fclose (handle);
 }
 
-static char parse_key[512];
-static char * parse_value;
-
-static void parse_next (FILE * handle)
-{
-    parse_value = nullptr;
-
-    if (! fgets (parse_key, sizeof parse_key, handle))
-        return;
-
-    char * space = strchr (parse_key, ' ');
-    if (! space)
-        return;
-
-    * space = 0;
-    parse_value = space + 1;
-
-    char * newline = strchr (parse_value, '\n');
-    if (newline)
-        * newline = 0;
-}
-
-static bool parse_integer (const char * key, int * value)
-{
-    return (parse_value && ! strcmp (parse_key, key) && sscanf (parse_value, "%d", value) == 1);
-}
-
-static String parse_string (const char * key)
-{
-    return (parse_value && ! strcmp (parse_key, key)) ? String (parse_value) : String ();
-}
-
-static void transport_plugin_parse (PluginHandle * plugin, FILE * handle)
+static void transport_plugin_parse (PluginHandle * plugin, TextParser & parser)
 {
     while (1)
     {
-        String value = parse_string ("scheme");
+        String value = parser.get_str ("scheme");
         if (! value)
             break;
 
         plugin->schemes.append (std::move (value));
-        parse_next (handle);
+        parser.next ();
     }
 }
 
-static void playlist_plugin_parse (PluginHandle * plugin, FILE * handle)
+static void playlist_plugin_parse (PluginHandle * plugin, TextParser & parser)
 {
     while (1)
     {
-        String value = parse_string ("ext");
+        String value = parser.get_str ("ext");
         if (! value)
             break;
 
         plugin->exts.append (std::move (value));
-        parse_next (handle);
+        parser.next ();
     }
 
-    if (parse_integer ("saves", & plugin->can_save))
-        parse_next (handle);
+    if (parser.get_int ("saves", plugin->can_save))
+        parser.next ();
 }
 
-static void input_plugin_parse (PluginHandle * plugin, FILE * handle)
+static void input_plugin_parse (PluginHandle * plugin, TextParser & parser)
 {
     for (auto key : aud::range<InputKey> ())
     {
         while (1)
         {
-            String value = parse_string (input_key_names[key]);
+            String value = parser.get_str (input_key_names[key]);
             if (! value)
                 break;
 
             plugin->keys[key].append (std::move (value));
-            parse_next (handle);
+            parser.next ();
         }
     }
 
-    if (parse_integer ("subtunes", & plugin->has_subtunes))
-        parse_next (handle);
-    if (parse_integer ("writes", & plugin->writes_tag))
-        parse_next (handle);
+    if (parser.get_int ("subtunes", plugin->has_subtunes))
+        parser.next ();
+    if (parser.get_int ("writes", plugin->writes_tag))
+        parser.next ();
 }
 
-static bool plugin_parse (FILE * handle)
+static bool plugin_parse (TextParser & parser)
 {
     PluginType type;
     String path;
@@ -300,7 +282,7 @@ static bool plugin_parse (FILE * handle)
     for (auto type2 : aud::range<PluginType> ())
     {
         type = type2;
-        if ((path = parse_string (plugin_type_names[type2])))
+        if ((path = parser.get_str (plugin_type_names[type2])))
             break;
     }
 
@@ -311,57 +293,61 @@ static bool plugin_parse (FILE * handle)
     if (! basename)
         return false;
 
-    parse_next (handle);
+    parser.next ();
 
     int timestamp;
-    if (! parse_integer ("stamp", & timestamp))
+    if (! parser.get_int ("stamp", timestamp))
         return false;
 
-    parse_next (handle);
+    parser.next ();
 
-    int version = 0;
-    if (parse_integer ("version", & version))
-        parse_next (handle);
+    int version = 0, flags = 0;
+    if (parser.get_int ("version", version))
+        parser.next ();
+    if (parser.get_int ("flags", flags))
+        parser.next ();
+
+    auto plugin = new PluginHandle (basename, String (), false, timestamp,
+     version, flags, type, nullptr);
 
-    auto plugin = new PluginHandle (basename, String (), false, timestamp, version, type, nullptr);
     plugins[type].append (plugin);
 
-    plugin->name = parse_string ("name");
+    plugin->name = parser.get_str ("name");
     if (plugin->name)
-        parse_next (handle);
+        parser.next ();
 
-    plugin->domain = parse_string ("domain");
+    plugin->domain = parser.get_str ("domain");
     if (plugin->domain)
-        parse_next (handle);
+        parser.next ();
 
-    if (parse_integer ("priority", & plugin->priority))
-        parse_next (handle);
-    if (parse_integer ("about", & plugin->has_about))
-        parse_next (handle);
-    if (parse_integer ("config", & plugin->has_configure))
-        parse_next (handle);
+    if (parser.get_int ("priority", plugin->priority))
+        parser.next ();
+    if (parser.get_int ("about", plugin->has_about))
+        parser.next ();
+    if (parser.get_int ("config", plugin->has_configure))
+        parser.next ();
 
     int enabled;
-    if (parse_integer ("enabled", & enabled))
+    if (parser.get_int ("enabled", enabled))
     {
         plugin->enabled = (PluginEnabled) enabled;
-        parse_next (handle);
+        parser.next ();
     }
 
     if (type == PluginType::Transport)
-        transport_plugin_parse (plugin, handle);
+        transport_plugin_parse (plugin, parser);
     else if (type == PluginType::Playlist)
-        playlist_plugin_parse (plugin, handle);
+        playlist_plugin_parse (plugin, parser);
     else if (type == PluginType::Input)
-        input_plugin_parse (plugin, handle);
+        input_plugin_parse (plugin, parser);
 
     return true;
 }
 
 /* try to migrate enabled status from another version */
-static void parse_plugins_fallback (FILE * handle)
+static void parse_plugins_fallback (TextParser & parser)
 {
-    for (; parse_value; parse_next (handle))
+    for (; ! parser.eof (); parser.next ())
     {
         PluginType type;
         String path;
@@ -370,7 +356,7 @@ static void parse_plugins_fallback (FILE * handle)
         for (auto type2 : aud::range<PluginType> ())
         {
             type = type2;
-            if ((path = parse_string (plugin_type_names[type2])))
+            if ((path = parser.get_str (plugin_type_names[type2])))
                 break;
         }
 
@@ -381,19 +367,19 @@ static void parse_plugins_fallback (FILE * handle)
         if (! basename)
             continue;
 
-        parse_next (handle);
+        parser.next ();
 
-        for (; parse_value; parse_next (handle))
+        for (; ! parser.eof (); parser.next ())
         {
-            if (parse_integer ("enabled", & enabled))
+            if (parser.get_int ("enabled", enabled))
                 break;
         }
 
-        if (! parse_value)
+        if (parser.eof ())
             return;
 
         // setting timestamp to zero forces a rescan
-        auto plugin = new PluginHandle (basename, String (), false, 0, 0, type, nullptr);
+        auto plugin = new PluginHandle (basename, String (), false, 0, 0, 0, type, nullptr);
         plugins[type].append (plugin);
         plugin->enabled = (PluginEnabled) enabled;
     }
@@ -405,27 +391,27 @@ void plugin_registry_load ()
     if (! handle)
         return;
 
-    parse_next (handle);
+    TextParser parser (handle);
 
     int format;
-    if (! parse_integer ("format", & format))
+    if (! parser.get_int ("format", format))
         goto ERR;
 
-    parse_next (handle);
+    parser.next ();
 
     if (format == FORMAT)
     {
-        while (plugin_parse (handle))
+        while (plugin_parse (parser))
             continue;
     }
     else if (format >= MIN_FORMAT && format < FORMAT + 10)
-        parse_plugins_fallback (handle);
+        parse_plugins_fallback (parser);
 
 ERR:
     fclose (handle);
 }
 
-static int plugin_compare (PluginHandle * const & a, PluginHandle * const & b, void *)
+static int plugin_compare (PluginHandle * const & a, PluginHandle * const & b)
 {
     if (a->type < b->type)
         return -1;
@@ -457,7 +443,7 @@ void plugin_registry_prune ()
 
     auto check_incompatible = [] (PluginHandle * plugin)
     {
-        if (plugin_check_flags (plugin->version))
+        if (plugin_check_flags (plugin->flags))
             return false;
 
         AUDINFO ("Incompatible plugin flags: %s\n", (const char *) plugin->basename);
@@ -467,7 +453,7 @@ void plugin_registry_prune ()
     for (auto type : aud::range<PluginType> ())
     {
         plugins[type].remove_if (check_not_found);
-        plugins[type].sort (plugin_compare, nullptr);
+        plugins[type].sort (plugin_compare);
         compatible[type].insert (plugins[type].begin (), 0, plugins[type].len ());
         compatible[type].remove_if (check_incompatible);
     }
@@ -499,6 +485,7 @@ static void plugin_get_info (PluginHandle * plugin, bool is_new)
     Plugin * header = plugin->header;
 
     plugin->version = header->version;
+    plugin->flags = header->info.flags;
     plugin->name = String (header->info.name);
     plugin->domain = String (header->info.domain);
     plugin->has_about = (bool) header->info.about;
@@ -580,6 +567,7 @@ void plugin_register (const char * path, int timestamp)
             plugin->timestamp = timestamp;
 
             plugin_get_info (plugin, false);
+            modified = true;
         }
     }
     else
@@ -590,10 +578,11 @@ void plugin_register (const char * path, int timestamp)
             return;
 
         plugin = new PluginHandle (basename, path, true, timestamp,
-         header->version, header->type, header);
+         header->version, header->info.flags, header->type, header);
         plugins[plugin->type].append (plugin);
 
         plugin_get_info (plugin, true);
+        modified = true;
     }
 }
 
@@ -680,6 +669,7 @@ void plugin_set_enabled (PluginHandle * plugin, PluginEnabled enabled)
 {
     plugin->enabled = enabled;
     plugin_call_watches (plugin);
+    modified = true;
 }
 
 void plugin_set_failed (PluginHandle * plugin)
