@@ -98,6 +98,20 @@ inline T from_ptr (void * v)
     return u.t;
 }
 
+// Move-assignment implemented via move-constructor
+// ================================================
+
+template<class T>
+T & move_assign (T & a, T && b)
+{
+    if (& a != & b)
+    {
+        a.~T ();
+        new (& a) T (std::move (b));
+    }
+    return a;
+}
+
 // Function wrappers (or "casts") for interaction with C-style APIs
 // ================================================================
 
@@ -114,6 +128,9 @@ void typed_func (T * obj)
     { func (obj); }
 
 template<class T, void (T::* func) ()>
+static void obj_member (void * obj)
+    { (((T *) obj)->* func) (); }
+template<class T, void (T::* func) () const>
 static void obj_member (void * obj)
     { (((T *) obj)->* func) (); }
 
