@@ -46,7 +46,7 @@ enum CategoryViewCols {
 };
 
 struct Category {
-    const char * icon_path;
+    const char * icon;
     const char * name;
 };
 
@@ -73,16 +73,19 @@ enum {
     CATEGORY_NETWORK,
     CATEGORY_PLAYLIST,
     CATEGORY_SONG_INFO,
-    CATEGORY_PLUGINS
+    CATEGORY_PLUGINS,
+    CATEGORY_ADVANCED
 };
 
+/* keep this in sync with the list in load_fallback_icons (init.cc) */
 static const Category categories[] = {
-    { "appearance.png", N_("Appearance") },
-    { "audio.png", N_("Audio") },
-    { "connectivity.png", N_("Network") },
-    { "playlist.png", N_("Playlist")} ,
-    { "info.png", N_("Song Info") },
-    { "plugins.png", N_("Plugins") }
+    { "applications-graphics", N_("Appearance") },
+    { "audio-volume-medium", N_("Audio") },
+    { "applications-internet", N_("Network") },
+    { "audio-x-generic", N_("Playlist")} ,
+    { "dialog-information", N_("Song Info") },
+    { "applications-system", N_("Plugins") },
+    { "preferences-system", N_("Advanced") }
 };
 
 static const PluginCategory plugin_categories[] = {
@@ -300,10 +303,9 @@ static const PreferencesWidget playlist_page_widgets[] = {
     WidgetCheck (N_("Show hours separately (1:30:00 vs. 90:00)"),
         WidgetBool (0, "show_hours", send_title_change)),
     WidgetCustomGTK (create_titlestring_table),
-    WidgetLabel (N_("<b>Compatibility</b>")),
-    WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
-        WidgetBool (0, "convert_backslash")),
-    WidgetTable ({{chardet_elements}})
+    WidgetLabel (N_("<b>Export</b>")),
+    WidgetCheck (N_("Use relative paths when possible"),
+        WidgetBool (0, "export_relative_paths"))
 };
 
 static const PreferencesWidget song_info_page_widgets[] = {
@@ -329,8 +331,20 @@ static const PreferencesWidget song_info_page_widgets[] = {
         WIDGET_CHILD),
     WidgetCheck (N_("Show time scale for current song"),
         WidgetBool (0, "filepopup_showprogressbar"),
-        WIDGET_CHILD),
-    WidgetLabel (N_("<b>Advanced</b>")),
+        WIDGET_CHILD)
+};
+
+static const PreferencesWidget advanced_page_widgets[] = {
+    WidgetLabel (N_("<b>Compatibility</b>")),
+    WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
+        WidgetBool (0, "convert_backslash")),
+    WidgetTable ({{chardet_elements}}),
+    WidgetLabel (N_("<b>Playlist</b>")),
+    WidgetCheck (N_("Add folders recursively"),
+        WidgetBool (0, "recurse_folders")),
+    WidgetCheck (N_("Add folders nested within playlist files"),
+        WidgetBool (0, "folders_in_playlist")),
+    WidgetLabel (N_("<b>Metadata</b>")),
     WidgetCheck (N_("Guess missing metadata from file path"),
         WidgetBool (0, "metadata_fallbacks")),
     WidgetCheck (N_("Do not load metadata for songs until played"),
@@ -470,7 +484,8 @@ static void fill_category_list (GtkTreeView * treeview, GtkNotebook * notebook)
      GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT);
     gtk_tree_view_set_model (treeview, (GtkTreeModel *) store);
 
-    const char * data_dir = aud_get_path (AudPath::DataDir);
+    GtkIconTheme * icon_theme = gtk_icon_theme_get_default ();
+    int icon_size = audgui_to_native_dpi (48);
 
     for (const Category & category : categories)
     {
@@ -482,8 +497,8 @@ static void fill_category_list (GtkTreeView * treeview, GtkNotebook * notebook)
         gtk_list_store_set (store, & iter, CATEGORY_VIEW_COL_NAME,
          gettext (category.name), -1);
 
-        StringBuf path = filename_build ({data_dir, "images", category.icon_path});
-        AudguiPixbuf img (gdk_pixbuf_new_from_file (path, nullptr));
+        AudguiPixbuf img (gtk_icon_theme_load_icon (icon_theme,
+         category.icon, icon_size, (GtkIconLookupFlags) 0, nullptr));
 
         if (img)
             gtk_list_store_set (store, & iter, CATEGORY_VIEW_COL_ICON, img.get (), -1);
@@ -747,7 +762,7 @@ static void record_update (void * = nullptr, void * = nullptr)
     {
         gtk_widget_set_sensitive (record_checkbox, false);
         gtk_button_set_label ((GtkButton *) record_checkbox,
-         str_printf (_("No audio recording plugin available")));
+         _("No audio recording plugin available"));
         gtk_toggle_button_set_active ((GtkToggleButton *) record_checkbox, false);
         gtk_widget_set_sensitive (record_config_button, false);
         gtk_widget_set_sensitive (record_about_button, false);
@@ -782,6 +797,13 @@ static void create_plugin_category ()
          plugin_view_new (category.type), gtk_label_new (_(category.name)));
 }
 
+static void create_advanced_category ()
+{
+    GtkWidget * advanced_page_vbox = gtk_vbox_new (false, 0);
+    audgui_create_widgets (advanced_page_vbox, advanced_page_widgets);
+    gtk_container_add ((GtkContainer *) category_notebook, advanced_page_vbox);
+}
+
 static void destroy_cb ()
 {
     hook_dissociate ("enable record", record_update);
@@ -834,6 +856,7 @@ static void create_prefs_window ()
     create_playlist_category ();
     create_song_info_category ();
     create_plugin_category ();
+    create_advanced_category ();
 
     GtkWidget * hseparator = gtk_hseparator_new ();
     gtk_box_pack_start ((GtkBox *) vbox, hseparator, false, false, 6);
