@@ -102,7 +102,7 @@ EXPORT void cleanup ()
 
     aboutwindow_hide ();
     equalizer_hide ();
-    infopopup_hide ();
+    infopopup_hide_now ();
     infowin_hide ();
     log_inspector_hide ();
     prefswin_hide ();
