@@ -21,30 +21,33 @@
 #define LIBAUDQT_IFACE_H
 
 #include <libaudcore/plugin.h>
+#include <libaudqt/export.h>
 #include <libaudqt/libaudqt.h>
 #include <libaudqt/menu.h>
 
 namespace audqt {
 
-class QtIfacePlugin : public IfacePlugin
+class LIBAUDQT_PUBLIC QtIfacePlugin : public IfacePlugin
 {
 public:
     constexpr QtIfacePlugin (PluginInfo info) : IfacePlugin (info) {}
 
     void show_about_window () { aboutwindow_show (); }
     void hide_about_window () { aboutwindow_hide (); }
-    void show_filebrowser (bool open) { fileopener_show (open ? FileMode::Open : FileMode::Add); }
+
+    void show_filebrowser (bool open)
+        { fileopener_show (open ? FileMode::Open : FileMode::Add); }
+
     void hide_filebrowser () {}
     void show_jump_to_song () {}
     void hide_jump_to_song () {}
     void show_prefs_window () { prefswin_show (); }
     void hide_prefs_window () { prefswin_hide (); }
-    void plugin_menu_add (AudMenuID id, void func (), const char * name, const char * icon) {
-        menu_add (id, func, name, icon);
-    }
-    void plugin_menu_remove (AudMenuID id, void func ()) {
-        menu_remove (id, func);
-    }
+
+    void plugin_menu_add (AudMenuID id, void func (), const char * name, const char * icon)
+        { menu_add (id, func, name, icon); }
+    void plugin_menu_remove (AudMenuID id, void func ())
+        { menu_remove (id, func); }
 };
 
 } // namespace audqt
