@@ -25,6 +25,7 @@
 #include <QString>
 #include <libaudcore/objects.h>
 
+class QIcon;
 class QLayout;
 class QBoxLayout;
 class QHBoxLayout;
@@ -46,6 +47,8 @@ enum class FileMode {
     OpenFolder,
     Add,
     AddFolder,
+    ImportPlaylist,
+    ExportPlaylist,
     count
 };
 
@@ -97,6 +100,8 @@ void run ();
 void quit ();
 void cleanup ();
 
+QIcon get_icon (const char * name);
+
 QHBoxLayout * make_hbox (QWidget * parent, int spacing = sizes.FourPt);
 QVBoxLayout * make_vbox (QWidget * parent, int spacing = sizes.FourPt);
 
@@ -129,10 +134,17 @@ void prefswin_show_plugin_page (PluginType type);
 void log_inspector_show ();
 void log_inspector_hide ();
 
-/* art.cc */
+/* art-qt.cc */
+QImage art_request (const char * filename, bool * queued = nullptr);
+QPixmap art_scale (const QImage & image, unsigned int w, unsigned int h, bool want_hidpi = true);
 QPixmap art_request (const char * filename, unsigned int w, unsigned int h, bool want_hidpi = true);
 QPixmap art_request_current (unsigned int w, unsigned int h, bool want_hidpi = true);
 
+/* infopopup-qt.cc */
+void infopopup_show (Playlist playlist, int entry);
+void infopopup_show_current ();
+void infopopup_hide ();
+
 /* infowin.cc */
 void infowin_show (Playlist playlist, int entry);
 void infowin_show_current ();
