@@ -30,9 +30,12 @@
 #include <libaudcore/multihash.h>
 #include <libaudcore/runtime.h>
 #include <libaudtag/builtin.h>
+#include <libaudtag/util.h>
 
 #include "id3-common.h"
 
+#define MAX_TAG_SIZE 16777216  /* reject tags over 16 MB */
+
 enum
 {
     ID3_ALBUM = 0,
@@ -47,11 +50,11 @@ enum
     ID3_YEAR,
     ID3_GENRE,
     ID3_COMMENT,
-    ID3_PRIVATE,
     ID3_ENCODER,
     ID3_RECORDING_TIME,
     ID3_TXXX,
     ID3_RVA2,
+    ID3_APIC,
     ID3_TAGS_NO
 };
 
@@ -68,32 +71,13 @@ static const char * id3_frames[ID3_TAGS_NO] = {
     "TYER",
     "TCON",
     "COMM",
-    "PRIV",
     "TSSE",
     "TDRC",
     "TXXX",
-    "RVA2"
+    "RVA2",
+    "APIC"
 };
 
-/*
-static const unsigned char PRIMARY_CLASS_MUSIC[16] = {0xBC, 0x7D, 0x60, 0xD1, 0x23,
- 0xE3, 0xE2, 0x4B, 0x86, 0xA1, 0x48, 0xA4, 0x2A, 0x28, 0x44, 0x1E};
-static const unsigned char PRIMARY_CLASS_AUDIO[16] = {0x29, 0x0F, 0xCD, 0x01, 0x4E,
- 0xDA, 0x57, 0x41, 0x89, 0x7B, 0x62, 0x75, 0xD5, 0x0C, 0x4F, 0x11};
-static const unsigned char SECONDARY_CLASS_AUDIOBOOK[16] = {0xEB, 0x6B, 0x23, 0xE0,
- 0x81, 0xC2, 0xDE, 0x4E, 0xA3, 0x6D, 0x7A, 0xF7, 0x6A, 0x3D, 0x45, 0xB5};
-static const unsigned char SECONDARY_CLASS_SPOKENWORD[16] = {0x13, 0x2A, 0x17, 0x3A,
- 0xD9, 0x2B, 0x31, 0x48, 0x83, 0x5B, 0x11, 0x4F, 0x6A, 0x95, 0x94, 0x3F};
-static const unsigned char SECONDARY_CLASS_NEWS[16] = {0x9B, 0xDB, 0x77, 0x66, 0xA0,
- 0xE5, 0x63, 0x40, 0xA1, 0xAD, 0xAC, 0xEB, 0x52, 0x84, 0x0C, 0xF1};
-static const unsigned char SECONDARY_CLASS_TALKSHOW[16] = {0x67, 0x4A, 0x82, 0x1B,
- 0x80, 0x3F, 0x3E, 0x4E, 0x9C, 0xDE, 0xF7, 0x36, 0x1B, 0x0F, 0x5F, 0x1B};
-static const unsigned char SECONDARY_CLASS_GAMES_CLIP[16] = {0x68, 0x33, 0x03, 0x00,
- 0x09, 0x50, 0xC3, 0x4A, 0xA8, 0x20, 0x5D, 0x2D, 0x09, 0xA4, 0xE7, 0xC1};
-static const unsigned char SECONDARY_CLASS_GAMES_SONG[16] = {0x31, 0xF7, 0x4F, 0xF2,
- 0xFC, 0x96, 0x0F, 0x4D, 0xA2, 0xF5, 0x5A, 0x34, 0x83, 0x68, 0x2B, 0x1A};
-*/
-
 #pragma pack(push) /* must be byte-aligned */
 #pragma pack(1)
 struct ID3v2Header {
@@ -175,6 +159,8 @@ static bool validate_header (ID3v2Header * header, bool is_footer)
         return false;
 
     header->size = unsyncsafe32 (FROM_BE32 (header->size));
+    if (header->size > MAX_TAG_SIZE)
+        return false;
 
     AUDDBG ("Found ID3v2 %s:\n", is_footer ? "footer" : "header");
     AUDDBG (" magic = %.3s\n", header->magic);
@@ -194,8 +180,7 @@ static bool read_header (VFSFile & handle, int * version, bool *
     if (handle.fseek (0, VFS_SEEK_SET))
         return false;
 
-    if (handle.fread (& header, 1, sizeof (ID3v2Header)) != sizeof
-     (ID3v2Header))
+    if (handle.fread (& header, 1, sizeof (ID3v2Header)) != sizeof (ID3v2Header))
         return false;
 
     if (validate_header (& header, false))
@@ -210,8 +195,7 @@ static bool read_header (VFSFile & handle, int * version, bool *
             if (handle.fseek (header.size, VFS_SEEK_CUR))
                 return false;
 
-            if (handle.fread (& footer, 1, sizeof (ID3v2Header)) != sizeof
-             (ID3v2Header))
+            if (handle.fread (& footer, 1, sizeof (ID3v2Header)) != sizeof (ID3v2Header))
                 return false;
 
             if (! validate_header (& footer, true))
@@ -235,8 +219,7 @@ static bool read_header (VFSFile & handle, int * version, bool *
         if (handle.fseek (end - sizeof (ID3v2Header), VFS_SEEK_SET))
             return false;
 
-        if (handle.fread (& footer, 1, sizeof (ID3v2Header)) != sizeof
-         (ID3v2Header))
+        if (handle.fread (& footer, 1, sizeof (ID3v2Header)) != sizeof (ID3v2Header))
             return false;
 
         if (! validate_header (& footer, true))
@@ -399,7 +382,7 @@ static void read_all_frames (const Index<char> & data, int version, FrameDict &
     }
 }
 
-static bool write_frame (int fd, const GenericFrame & frame, int version, int * frame_size)
+static bool write_frame (VFSFile & file, const GenericFrame & frame, int version, int * frame_size)
 {
     AUDDBG ("Writing frame %s, size %d\n", (const char *) frame.key, frame.len ());
 
@@ -414,44 +397,35 @@ static bool write_frame (int fd, const GenericFrame & frame, int version, int *
     header.size = TO_BE32 (size);
     header.flags = 0;
 
-    if (write (fd, & header, sizeof (ID3v2FrameHeader)) != sizeof (ID3v2FrameHeader))
+    if (file.fwrite (& header, 1, sizeof (ID3v2FrameHeader)) != sizeof (ID3v2FrameHeader))
         return false;
 
-    if (write (fd, & frame[0], frame.len ()) != frame.len ())
+    if (file.fwrite (& frame[0], 1, frame.len ()) != frame.len ())
         return false;
 
     * frame_size = sizeof (ID3v2FrameHeader) + frame.len ();
     return true;
 }
 
-struct WriteState {
-    int fd;
-    int version;
-    int written_size;
-};
-
-static void write_frame_list (const String & key, FrameList & list, void * user)
+static int write_all_frames (VFSFile & file, FrameDict & dict, int version)
 {
-    WriteState * state = (WriteState *) user;
+    int written_size = 0;
 
-    for (const GenericFrame & frame : list)
+    dict.iterate ([&] (const String & key, FrameList & list)
     {
-        int size;
-        if (write_frame (state->fd, frame, state->version, & size))
-            state->written_size += size;
-    }
+        for (const GenericFrame & frame : list)
+        {
+            int size;
+            if (write_frame (file, frame, version, & size))
+                written_size += size;
+        }
+    });
+
+    AUDDBG ("Total frame bytes written = %d.\n", written_size);
+    return written_size;
 }
 
-static int write_all_frames (int fd, FrameDict & dict, int version)
-{
-    WriteState state = {fd, version, 0};
-    dict.iterate (write_frame_list, & state);
-
-    AUDDBG ("Total frame bytes written = %d.\n", state.written_size);
-    return state.written_size;
-}
-
-static bool write_header (int fd, int version, int size)
+static bool write_header (VFSFile & file, int version, int size)
 {
     ID3v2Header header;
 
@@ -461,14 +435,12 @@ static bool write_header (int fd, int version, int size)
     header.flags = 0;
     header.size = TO_BE32 (syncsafe32 (size));
 
-    return write (fd, & header, sizeof (ID3v2Header)) == sizeof (ID3v2Header);
+    return file.fwrite (& header, 1, sizeof (ID3v2Header)) == sizeof (ID3v2Header);
 }
 
 static int get_frame_id (const char * key)
 {
-    int id;
-
-    for (id = 0; id < ID3_TAGS_NO; id ++)
+    for (int id = 0; id < ID3_TAGS_NO; id ++)
     {
         if (! strcmp (key, id3_frames[id]))
             return id;
@@ -477,50 +449,6 @@ static int get_frame_id (const char * key)
     return -1;
 }
 
-#if 0
-static void decode_private_info (Tuple & tuple, const unsigned char * data, int size)
-{
-    char * text = g_strndup ((const char *) data, size);
-
-    if (!strncmp(text, "WM/", 3))
-    {
-        char *separator = strchr(text, 0);
-        if (separator == nullptr)
-            goto DONE;
-
-        char * value = separator + 1;
-        if (!strncmp(text, "WM/MediaClassPrimaryID", 22))
-        {
-            if (!memcmp(value, PRIMARY_CLASS_MUSIC, 16))
-                tuple.set_str (-1, "media-class", "Music");
-            if (!memcmp(value, PRIMARY_CLASS_AUDIO, 16))
-                tuple.set_str (-1, "media-class", "Audio (non-music)");
-        } else if (!strncmp(text, "WM/MediaClassSecondaryID", 24))
-        {
-            if (!memcmp(value, SECONDARY_CLASS_AUDIOBOOK, 16))
-                tuple.set_str (-1, "media-class", "Audio Book");
-            if (!memcmp(value, SECONDARY_CLASS_SPOKENWORD, 16))
-                tuple.set_str (-1, "media-class", "Spoken Word");
-            if (!memcmp(value, SECONDARY_CLASS_NEWS, 16))
-                tuple.set_str (-1, "media-class", "News");
-            if (!memcmp(value, SECONDARY_CLASS_TALKSHOW, 16))
-                tuple.set_str (-1, "media-class", "Talk Show");
-            if (!memcmp(value, SECONDARY_CLASS_GAMES_CLIP, 16))
-                tuple.set_str (-1, "media-class", "Game Audio (clip)");
-            if (!memcmp(value, SECONDARY_CLASS_GAMES_SONG, 16))
-                tuple.set_str (-1, "media-class", "Game Soundtrack");
-        } else {
-            AUDDBG("Unrecognised tag %s (Windows Media) ignored\n", text);
-        }
-    } else {
-        AUDDBG("Unable to decode private data, skipping: %s\n", text);
-    }
-
-DONE:
-    g_free (text);
-}
-#endif
-
 static GenericFrame & add_generic_frame (int id, int size, FrameDict & dict)
 {
     String key (id3_frames[id]);
@@ -598,13 +526,10 @@ static void add_frameFromTupleStr (const Tuple & tuple, Tuple::Field field,
 static void add_frameFromTupleInt (const Tuple & tuple, Tuple::Field field,
  int id3_field, FrameDict & dict)
 {
-    if (tuple.get_value_type (field) != Tuple::Int)
-    {
+    if (tuple.get_value_type (field) == Tuple::Int)
+        add_text_frame (id3_field, int_to_str (tuple.get_int (field)), dict);
+    else
         remove_frame (id3_field, dict);
-        return;
-    }
-
-    add_text_frame (id3_field, int_to_str (tuple.get_int (field)), dict);
 }
 
 bool ID3v24TagModule::can_handle_file (VFSFile & handle)
@@ -617,7 +542,7 @@ bool ID3v24TagModule::can_handle_file (VFSFile & handle)
      & data_size, & footer_size);
 }
 
-bool ID3v24TagModule::read_tag (Tuple & tuple, VFSFile & handle)
+bool ID3v24TagModule::read_tag (VFSFile & handle, Tuple & tuple, Index<char> * image)
 {
     int version, header_size, data_size, footer_size;
     bool syncsafe;
@@ -628,6 +553,7 @@ bool ID3v24TagModule::read_tag (Tuple & tuple, VFSFile & handle)
         return false;
 
     Index<char> data = read_tag_data (handle, data_size, syncsafe);
+    FrameList rva_frames;
 
     for (const char * pos = data.begin (); pos < data.end (); )
     {
@@ -676,13 +602,15 @@ bool ID3v24TagModule::read_tag (Tuple & tuple, VFSFile & handle)
           case ID3_COMMENT:
             id3_decode_comment (tuple, & frame[0], frame.len ());
             break;
-#if 0
-          case ID3_PRIVATE:
-            decode_private_info (tuple, & frame[0], frame.len ());
+          case ID3_TXXX:
+            id3_decode_txxx (tuple, & frame[0], frame.len ());
             break;
-#endif
           case ID3_RVA2:
-            id3_decode_rva (tuple, & frame[0], frame.len ());
+            rva_frames.append (std::move (frame));
+            break;
+          case ID3_APIC:
+            if (image)
+                * image = id3_decode_picture (& frame[0], frame.len ());
             break;
           default:
             AUDDBG ("Ignoring unsupported ID3 frame %s.\n", (const char *) frame.key);
@@ -692,40 +620,17 @@ bool ID3v24TagModule::read_tag (Tuple & tuple, VFSFile & handle)
         pos += frame_size;
     }
 
+    /* only decode RVA2 frames if Replay Gain was not found in TXXX frames */
+    if (! tuple.is_set (Tuple::GainDivisor) && ! tuple.is_set (Tuple::PeakDivisor))
+    {
+        for (auto & rva : rva_frames)
+            id3_decode_rva (tuple, & rva[0], rva.len ());
+    }
+
     return true;
 }
 
-Index<char> ID3v24TagModule::read_image (VFSFile & handle)
-{
-    Index<char> buf;
-    int version, header_size, data_size, footer_size;
-    bool syncsafe;
-    int64_t offset;
-
-    if (! read_header (handle, & version, & syncsafe, & offset, & header_size,
-     & data_size, & footer_size))
-        return buf;
-
-    Index<char> data = read_tag_data (handle, data_size, syncsafe);
-
-    for (const char * pos = data.begin (); pos < data.end (); )
-    {
-        int frame_size;
-        GenericFrame frame;
-
-        if (! read_frame (pos, data.end () - pos, version, & frame_size, frame))
-            break;
-
-        if (! strcmp (frame.key, "APIC"))
-            buf = id3_decode_picture (& frame[0], frame.len ());
-
-        pos += frame_size;
-    }
-
-    return buf;
-}
-
-bool ID3v24TagModule::write_tag (const Tuple & tuple, VFSFile & f)
+bool ID3v24TagModule::write_tag (VFSFile & f, const Tuple & tuple)
 {
     int version = 3;
     int header_size, data_size, footer_size;
@@ -754,26 +659,26 @@ bool ID3v24TagModule::write_tag (const Tuple & tuple, VFSFile & f)
     int64_t mp3_offset = offset ? 0 : header_size + data_size + footer_size;
     int64_t mp3_size = offset ? offset : -1;
 
-    TempFile temp = TempFile ();
-    if (! open_temp_file_for (& temp, f))
+    auto temp = VFSFile::tmpfile ();
+    if (! temp)
         return false;
 
     /* write empty header (will be overwritten later) */
-    if (! write_header (temp.fd, version, 0))
+    if (! write_header (temp, version, 0))
         return false;
 
     /* write tag data */
-    data_size = write_all_frames (temp.fd, dict, version);
+    data_size = write_all_frames (temp, dict, version);
 
     /* copy non-tag data */
-    if (! copy_region_to_temp_file (& temp, f, mp3_offset, mp3_size))
+    if (f.fseek (mp3_offset, VFS_SEEK_SET) < 0 || ! temp.copy_from (f, mp3_size))
         return false;
 
     /* go back to beginning and write real header */
-    if (lseek (temp.fd, 0, SEEK_SET) < 0 || ! write_header (temp.fd, version, data_size))
+    if (temp.fseek (0, VFS_SEEK_SET) < 0 || ! write_header (temp, version, data_size))
         return false;
 
-    if (! replace_with_temp_file (& temp, f))
+    if (! f.replace_with (temp))
         return false;
 
     return true;
